/*
 Mentorenwahl: A fullstack application for assigning mentors to students based on their whishes.
 Copyright (C) 2022  Dominic Grimm
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
-- +micrate Up
-- SQL in section ' Up ' is executed when this migration is applied
CREATE TYPE user_roles AS ENUM ('student', 'teacher');

CREATE TABLE users(
  id SERIAL PRIMARY KEY,
  username TEXT UNIQUE NOT NULL,
  role user_roles NOT NULL,
  admin BOOLEAN NOT NULL,
  jti uuid UNIQUE
);

CREATE TABLE teachers(
  id SERIAL PRIMARY KEY,
  user_id INT NOT NULL UNIQUE REFERENCES users(id),
  max_students INT NOT NULL
);

CREATE TABLE students(
  id SERIAL PRIMARY KEY,
  user_id INT NOT NULL UNIQUE REFERENCES users(id)
);

CREATE TABLE votes(
  id SERIAL PRIMARY KEY,
  student_id INT NOT NULL UNIQUE REFERENCES students(id)
);

CREATE TABLE teacher_votes(
  id SERIAL PRIMARY KEY,
  vote_id INT NOT NULL REFERENCES votes(id),
  teacher_id INT NOT NULL REFERENCES teachers(id),
  priority INT NOT NULL
);

CREATE TABLE assignments(
  id SERIAL PRIMARY KEY,
  student_id INT NOT NULL REFERENCES students(id),
  teacher_id INT NOT NULL REFERENCES teachers(id)
);

-- +micrate Down
-- SQL section ' Down ' is executed when this migration is rolled back
DROP TABLE teacher_votes;

DROP TABLE votes;

DROP TABLE admins;

DROP TABLE teachers;

DROP TABLE students;

DROP TABLE users;

DROP TYPE user_roles;