# Mentorenwahl: A fullstack application for assigning mentors to students based on their whishes.
# Copyright (C) 2022  Dominic Grimm
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

require "clear"
require "log"
require "db"
require "retriable"

require "./db/*"

module Backend
  # Database model definitions
  module Db
    extend self

    # Migration UIDs
    MIGRATIONS = {{ run("./macros/migrations.cr", "db/migrations/*.sql").stringify.split("\n") }}

    def init(severity = {% if !flag?(:release) %} ::Log::Severity::Debug {% else %} ::Log::Severity::Info {% end %}) : Nil
      ::Log.builder.bind "clear.*", severity, ::Log::IOBackend.new
      Retriable.retry(on: DB::ConnectionRefused, backoff: false) do
        Clear::SQL.init(Backend.config.db.url)
      end
    end

    def schema_up_to_date? : Bool
      MicrateDbVersion.query.last!.version_id == MIGRATIONS.last.to_i64
    end
  end
end
